-- File:     codersea/codersea-jellyfish.lua
-- Purpose:  CoderSea jellyfish support module
-- Credits:  See "codersea.md"
-- Licenses: See "LICENSE"

local spawnchance = codersea_chance_jellyfish
if    spawnchance < 300 then
      spawnchance = 300 end

local mobtype = "animal"
if codersea_monster_jellyfish then mobtype = "monster" end

mobs:register_mob ("codersea:jellyfish", {
    do_custom     = mob_custom    ,
    type          = mobtype       ,
    attack_type   = "dogfight"    ,
    damage        =  3            ,
    reach         =  1            ,
    hp_min        =  5            ,
    hp_max        = 10            ,
    armor         = 40            ,
    walk_velocity = 0.1           ,
    run_velocity  = 0.1           ,
    fly           = true          ,
    fly_in        = fly_water     ,
    fall_speed    =  0            ,
    view_range    =  2            ,
    water_damage  =  0            ,
    lava_damage   =  5            ,
    light_damage  =  0            ,
    visual        = "mesh"        ,

    mesh = "codersea_jellyfish.b3d" ,
    collisionbox = { -0.25, -0.25, -0.25, 0.25, 0.25, 0.25 } ,

    textures = {
        { "codersea_jellyfish.png" }
    },

    makes_footstep_sound = false,

    on_rightclick = function (self, clicker)
        mobs:capture_mob (self, clicker,
            80, 100, 0, true, nil)
    end
})

-- ===================================================================

mobs:spawn_specific ("codersea:jellyfish",
    {
        "codersea:water_source"  ,
        "default:water_source"   ,
        "default:water_gel"      ,
    } ,
    {
        "codersea:water_flowing" ,
        "codersea:water_source"  ,
        "default:water_flowing"  ,
        "default:water_source"   ,
    } ,
2, 25, 30, spawnchance, codersea_aoc_jellyfish, -31000, -1)

mobs:register_egg ("codersea:jellyfish",
    "Jellyfish"  , "codersea_jellyfish_item.png", 0)

-- ===================================================================

minetest.register_node ("codersea:jellyfish_spawner", {
    description = "Jellyfish spawner" ,
    tiles = { "jellyfish_spawner.png" } ,
    paramtype = "light" ,
    light_source = default.LIGHT_MAX ,
    paramtype2 = "facedir" ,
    groups = {cracky = 2} ,
    is_ground_content = false ,
    legacy_facedir_simple = true ,
    sounds = default.node_sound_defaults() ,
})

-- ===================================================================

minetest.register_abm ({
    catch_up  = false ,
    chance    = 1     ,
    interval  = 3     ,

    nodenames = {
        "codersea:jellyfish_spawner" ,
    } ,

    action    = function (pos, node)
        local entname = "codersea:jellyfish"
        local uppos   = { x = pos.x, y = pos.y, z = pos.z }
        uppos.y = uppos.y + math.random (2, 6)
        if not is_water (uppos) then return end
        if (mobs.count_mobs (uppos, entname) > 5) then return end
        minetest.add_entity (uppos, entname)
    end ,
})
