-- File:     codersea/codersea-shark.lua
-- Purpose:  CoderSea shark mob support module
-- Credits:  See "codersea.md"
-- Licenses: See "LICENSE"

-- local variables
local l_colors = {
    "#604000:175",  --brown
    "#ffffff:150",  --white
    "#404040:150",  --dark_grey
    "#a0a0a0:150"   --grey
}

local l_skins = {
    {"(codersea_shark_first.png^[colorize:"..l_colors[3]..")^(codersea_shark_second.png^[colorize:"..l_colors[4]..")^codersea_shark_third.png"},
    {"(codersea_shark_first.png^[colorize:"..l_colors[1]..")^(codersea_shark_second.png^[colorize:"..l_colors[2]..")^codersea_shark_third.png"},
    {"(codersea_shark_first.png^[colorize:"..l_colors[4]..")^(codersea_shark_second.png^[colorize:"..l_colors[2]..")^codersea_shark_third.png"}
}

local l_anims = {
    speed_normal = 24, speed_run =  24,
    stand_start  =  1, stand_end =  80,
    walk_start   = 80, walk_end  = 160,
    run_start    = 80, run_end   = 160,
}

local l_model           = "codersea_shark.b3d"
local l_egg_texture     = "codersea_shark_item.png"

local l_spawn_in        = {
    "codersea:water_source"       ,
    "codersea:water_flowing"      ,
    "codersea:water_gel"          ,
    "default:water_source"        ,
    "default:water_flowing"       ,
}

local l_spawn_near   = {
    "darkage:silt"           ,
    "default:sand"           ,
    "default:dirt"           ,
    "ethereal:dry_dirt"      ,
    "ethereal:green_dirt"    ,
    "ethereal:grove_dirt"    ,
    "ethereal:mushroom_dirt" ,
    "group:sand"             ,
    "group:soil"             ,
}

local spawnchance = codersea_chance_shark
if    spawnchance < 300 then
      spawnchance = 300 end

local mobtype = "animal"
if codersea_monster_shark then mobtype = "monster" end

mobs:register_mob ("codersea:lgshark", {
    do_custom   = mob_custom  ,
    type        = mobtype     ,
    attack_type = "dogfight"  ,
    damage      =  10         ,
    reach       =   3         ,
    hp_min      =  20         ,
    hp_max      =  25         ,
    armor       = 150         ,
    view_range  =   6         ,

    collisionbox = {-0.75, -0.5, -0.75, 0.75, 0.5, 0.75},
    visual = "mesh",
    mesh = l_model,
    textures = l_skins,
    makes_footstep_sound = false,
    walk_velocity = 4,
    run_velocity = 6,
    fly = true,
    fly_in = fly_water,
    fall_speed = 0,
    rotate = 270,
    water_damage = 0,
    lava_damage = 10,
    light_damage = 0,
    animation = l_anims,
})

mobs:spawn_specific ("codersea:lgshark",
    l_spawn_in, l_spawn_near,
    -1, 20, 30, spawnchance, codersea_aoc_shark, -31000, -1)

mobs:register_egg ("codersea:lgshark",
    "Large Shark", l_egg_texture, 0)

mobs:register_mob ("codersea:mdshark", {
    do_custom = mob_custom,
    type = mobtype,
    attack_type = "dogfight",
    damage = 8,
    reach = 2,
    hp_min = 15,
    hp_max = 20,
    armor = 125,
    collisionbox = {-0.57, -0.38, -0.57, 0.57, 0.38, 0.57},
    visual = "mesh",
    visual_size = {x=0.75, y=0.75},
    mesh = l_model,
    textures = l_skins,
    makes_footstep_sound = false,
    walk_velocity = 2,
    run_velocity = 4,
    fly = true,
    fly_in = fly_water,
    fall_speed = -1,
    rotate = 270,
    view_range = 6,
    water_damage = 0,
    lava_damage = 10,
    light_damage = 0,
    animation = l_anims
})

mobs:spawn_specific ("codersea:mdshark",
    l_spawn_in, l_spawn_near,
    -1, 20, 30, spawnchance, codersea_aoc_shark, -31000, -1)

mobs:register_egg ("codersea:mdshark",
    "Medium Shark", l_egg_texture, 0)

mobs:register_mob ("codersea:smshark", {
    do_custom = mob_custom,
    type = mobtype,
    attack_type = "dogfight",
    damage = 6,
    reach = 1,
    hp_min = 10,
    hp_max = 15,
    armor = 100,
    collisionbox = {-0.38, -0.25, -0.38, 0.38, 0.25, 0.38},
    visual = "mesh",
    visual_size = {x=0.5, y=0.5},
    mesh = l_model,
    textures = l_skins,
    makes_footstep_sound = false,
    walk_velocity = 2,
    run_velocity = 4,
    fly = true,
    fly_in = fly_water,
    fall_speed = -1,
    rotate = 270,
    view_range = 6,
    water_damage = 0,
    lava_damage = 10,
    light_damage = 0,
    animation = l_anims
})

mobs:spawn_specific ("codersea:smshark",
    l_spawn_in, l_spawn_near,
    -1, 20, 30, spawnchance, codersea_aoc_shark, -31000, -1)

mobs:register_egg ("codersea:smshark",
    "Small Shark", l_egg_texture, 0)
