-- File:     codersea/seagravel.lua
-- Purpose:  CoderSea sea-gravel support
-- Credits:  See "codersea.md"
-- Licenses: See "LICENSE"

-- ===================================================================

local function register_seagravel_craft (output, recipe)
    minetest.register_craft ({
        type   = "shapeless" ,
        output = output      ,
        recipe = recipe      ,
    })
end

-- ===================================================================

local function register_seagravel (suffix)
    local basename = "seagravel" .. suffix
    local esuffix  = suffix:gsub ("_", " ")
    local nodename = "codersea:" .. basename
    local img      = "codersea_" .. basename .. ".png"

    minetest.register_node (nodename, {
        description       = "Sea-gravel" .. esuffix       ,
        tiles             = { img }                       ,
        is_ground_content = true                          ,
        groups            = { crumbly=2, falling_node=1 } ,

        sounds = default.node_sound_dirt_defaults ({
            footstep = {
                name="default_gravel_footstep", gain=0.5
            } ,
            dug      = {
                name="default_gravel_footstep", gain=1.0
            } ,
        }) ,
    })

    if not codersea_disable_stairs then
        stairs.register_stair_and_slab (basename, nodename,
            { crumbly=2, falling_node=1 } ,
            { img }                       ,
            "Sea-gravel stair" .. esuffix ,
            "Sea-gravel slab"  .. esuffix ,

            default.node_sound_dirt_defaults ({
                footstep = {
                    name="default_gravel_footstep", gain=0.5
                } ,
                dug = {
                    name="default_gravel_footstep", gain=1.0
                } ,
            })
        )
    end

    if suffix:len() > 0 then
        local dye = ("dye" .. suffix):gsub ("_", ":")
        register_seagravel_craft (nodename ,
            { "codersea:seagravel", dye })
        register_seagravel_craft (nodename ,
            { "codersea:crushedwhite", "default:gravel" , dye })

    else
        register_seagravel_craft (nodename ,
            { "codersea:crushedwhite", "default:gravel" })
    end
end

-- ===================================================================

for _,color in pairs (seastone_colors) do
    if color:len() > 0 then color = "_" .. color end
    register_seagravel (color)
end
