-- File:     codersea/seastone.lua
-- Purpose:  CoderSea sea-stone support
-- Credits:  See "codersea.md"
-- Licenses: See "LICENSE"

-- ===================================================================

local function register_seastone_craft (output, recipe)
    minetest.register_craft ({
        type   = "shapeless" ,
        output = output      ,
        recipe = recipe      ,
    })
end

-- ===================================================================

local function register_seastone_cooking (output, recipe)
    minetest.register_craft ({
       type   = "cooking" ,
       output = output    ,
       recipe = recipe    ,
    })
end

-- ===================================================================

local function register_seastone    (suffix)
    local basename =   "seastone" .. suffix
    local esuffix  = suffix:gsub ("_", " ")
    local nodename = "codersea:" .. basename
    local img      = "codersea_" .. basename .. ".png"
    local cobble   = "codersea:seacobble" .. suffix

    table.insert (seastone_list, nodename)

    minetest.register_node (nodename, {
        description       = "Sea-stone" .. esuffix ,
        drop              = cobble                 ,
        groups            = { cracky=3, stone=1 }  ,
        is_ground_content = true                   ,
        sounds            = sounds_stone           ,
        tiles             = { img }                ,
    })

if not codersea_disable_stairs then
    stairs.register_stair_and_slab (basename, nodename,
        { cracky=3, stone=1, wrapwater=1 }        ,
        { img }                      ,
        "Sea-stone stair" .. esuffix ,
        "Sea-stone slab"  .. esuffix ,
        sounds_stone)
end
    if suffix:len() > 0 then
        local dye = ("dye" .. suffix):gsub ("_", ":")
        register_seastone_craft (nodename,
            { "codersea:seastone", dye })
        register_seastone_craft (nodename,
            { "codersea:crushedwhite", "default:stone", dye })
    else
        register_seastone_craft (nodename,
            { "codersea:crushedwhite", "default:stone" })
    end

    register_seastone_cooking (nodename, cobble)
end

-- ===================================================================

for  _,color in pairs (seastone_colors) do
    if color:len() > 0 then color = "_" .. color end
    register_seastone (color)
end
