-- Car. Descended from Sokomine version.

-- ===================================================================
-- Media license. Applies to model and associated textures.

-- CC-BY-NC-SA 3.0. Attribution: Melkor.

-- ===================================================================

local lcname     = "car"
local ucname     = "Car"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance = 120000       ,

    day_mode     = true         ,
    min_light    =     12       ,
    max_light    =     20       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    scale        = 1.00         ,
    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_grass"       ,
        "default:sand"                  ,
        "default:silver_sand"           ,
        "default:stone"                 ,
        "ethereal:green_dirt"           ,
        "ethereal:green_dirt_top"       ,
        "ethereal:grove_dirt"           ,
        "mg:dirt_with_dry_grass"        ,
        "noisegrid:grass"               ,
        "streets:asphalt"               ,
    } ,

    spawn_by     = {
    } ,

    add_egg   = true                    ,
    egg_image = "default_grass.png"     ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type    ,
    makes_footstep_sound = true     ,

    armor         = 200             ,
    attack_npcs   = false           ,
    attack_type   = "dogfight"      ,
    damage        =   4             ,
    fear_height   =   3             ,
    hp_max        =  20             ,
    hp_min        =   5             ,
    stepheight    = 1.2             ,
    jump_height   =   0             ,
    jump          = false           ,
    passive       = false           ,
    pushable      = true            ,
    reach         =   2             ,
    run_velocity  =   2             ,
    view_range    =   8             ,
    walk_velocity =   1             ,
    lava_damage   =   5             ,
    light_damage  =   0             ,
    water_damage  =   1             ,
    floats        =   0             ,
    runaway        = true           ,

    sounds = {
    } ,

    drops = {
    } ,

    follow          = { "codermobs:can_oil"     } ,

    animation = {
        walk_start  = 0 ,
        walk_end    = 1 ,
        stand_start = 0 ,
        stand_end   = 1 ,
    } ,

-- Original:
--  collisionbox  = { -1.85, -0.05, -0.50, 1.25, 0.70, 1.30 } ,

    collisionbox  = {
                        -1.0 , -1.00, -1.00,
                         1.0 ,  1.00,  1.00,
                    } ,

-- Rotation of the car and/or attached players is no longer needed be-
-- cause Oldcoder has modified the original model, in Blender, as fol-
-- lows: Move down 10 units. Rotate 270 degrees.

    mesh          = "codermobs_car.b3d"           ,
    visual        = "mesh"                        ,

-- These five  settings are needed,  for ridable or drivable mobs,  by
-- "mobs/mount.lua".

    driver_attach_at  = { x = 0, y =  2, z = 0 }  ,
    accel             = 3                         ,
    max_speed_forward = 7                         ,
    max_speed_reverse = 3                         ,

    textures = {
        { "codermobs_car_black_mesh.png"        } ,
        { "codermobs_car_blue_mesh.png"         } ,
        { "codermobs_car_brown_mesh.png"        } ,
        { "codermobs_car_cyan_mesh.png"         } ,
        { "codermobs_car_dark_green_mesh.png"   } ,
        { "codermobs_car_dark_grey_mesh.png"    } ,
        { "codermobs_car_green_mesh.png"        } ,
        { "codermobs_car_grey_mesh.png"         } ,
        { "codermobs_car_magenta_mesh.png"      } ,
        { "codermobs_car_orange_mesh.png"       } ,
        { "codermobs_car_pink_mesh.png"         } ,
        { "codermobs_car_red_mesh.png"          } ,
        { "codermobs_car_violet_mesh.png"       } ,
        { "codermobs_car_white_mesh.png"        } ,
        { "codermobs_car_yellow_mesh.png"       } ,
    } ,

    on_rightclick = function (self, clicker)
        if mobs:feed_tame (self, clicker, 8, true, true) then return end
        if mobs:protect (self, clicker) then return end

        if self.driver and clicker == self.driver then
            mobs.detach (clicker, { x=1, y=0, z=1 })
        elseif not self.driver then
            mobs.attach (self, clicker)
        end

        if mobs:capture_mob (self, clicker, 0, 5, 50, false, nil) then
            return
        end
    end ,

    do_custom = function (self, dtime)
        if self.driver then
            self.state = "stand"
            mobs.drive (self, nil, nil, false, dtime)
            return false
            end
        return true
    end ,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
