-- Dog. Descended from "pmobs" version.

-- ===================================================================

local lcname     = "dog"
local ucname     = "Dog"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance = 120000       ,

    day_mode     = true         ,
    min_light    =      0       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "npc"        ,

    spawn_nodes  = {
        "default:dirt_with_dry_grass"       ,
        "default:dirt_with_grass"           ,
        "default:dirt"                      ,
        "default:snow"                      ,
        "default:snowblock"                 ,
        "earthgen:dirt_with_dry_grass"      ,
        "ethereal:green_dirt"               ,
        "mg:dirt_with_dry_grass"            ,
        "noisegrid:grass"                   ,
    } ,

    add_egg   = true                        ,
    egg_image = "codermobs_fur_brown.png"   ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,

    passive       = false       ,
    pathfinding   = true        ,
    reach         =   1         ,
    damage        =   2         ,
    attack_type   = "dogfight"  ,

    hp_max = 5,
    collisionbox = {-0.18, 0.00, -0.18, 0.18, 0.92, 0.18},
    visual = "mesh",
    mesh = msname .. ".b3d",
    textures = {
        { msname .. ".png"},
    },
    makes_footstep_sound = true,
    sounds = {
        war_cry = msname .. "_attack" ,
    },
    view_range = 15,
    stepheight    = 1.2         ,
    jump_height   =   0         ,
    jump          = false       ,
    owner = "",
    order = "follow",

    walk_velocity = 0.5806,
    run_velocity = 5.2557,
    stepheight = 1.1,
    damage = 2,
    armor = 200,
    attacks_monsters = true,
    attack_type = "dogfight",

    drops = {
        { name="mobs:meat_raw", chance=1, min=2, max=3 } ,
    } ,

    specific_attack = codermobs.make_mob_list ({
        "chicken", "bunny",
    }) ,

    -- drawtype = "front",
    rotate = 180,
    water_damage = 1,
    floats = 0,
    runaway = true,
    lava_damage = 5,
    light_damage = 0,

    on_rightclick = function (self, clicker)
        local item = clicker:get_wielded_item()
        if item:get_name() == "mobs:meat_raw" then
            local hp = self.object:get_hp()
            if hp + 4 > self.hp_max then return end
            if not minetest.setting_getbool("creative_mode") then
                item:take_item()
                clicker:set_wielded_item(item)
            end
            self.object:set_hp(hp+4)
        else
            if self.owner == "" then
                self.owner = clicker:get_player_name()
            else
                local formspec = "size[8,4]"
                formspec = formspec .. "textlist[2.85,0;2.1,0.5;dialog;What can I do for you?]"
                formspec = formspec .. "button_exit[1,1;2,2;dfollow;follow]"
                formspec = formspec .. "button_exit[5,1;2,2;dstand;stand]"
                formspec = formspec .. "button_exit[0,2;4,4;dfandp;follow and protect]"
                formspec = formspec .. "button_exit[4,2;4,4;dsandp;stand and protect]"
                formspec = formspec .. "button_exit[1,2;2,2;dgohome; go home]"
                formspec = formspec .. "button_exit[5,2;2,2;dsethome; sethome]"

                minetest.show_formspec(clicker:get_player_name(), "order", formspec)

                minetest.register_on_player_receive_fields(function(clicker, formname, fields)
                    if fields.dfollow then
                        self.order = "follow"
                        self.attacks_monsters = false
                    end
                    if fields.dstand then
                        self.order = "stand"
                        self.attacks_monsters = false
                    end
                    if fields.dfandp then
                        self.order = "follow"
                        self.attacks_monsters = true
                    end
                    if fields.dsandp then
                        self.order = "stand"
                        self.attacks_monsters = true
                    end

                    -- 211004 RJK: This was probably incorrect
                    if false and fields.dsethome then
                        self.floats = self.object:getpos()
                    end

                    -- 211004 RJK: Work-around for a crash
                    if false and fields.dgohome then
                        if self.floats then
                            self.order = "stand"
                            self.object:setpos (self.floats)
                        end
                    end
                end)
            end
        end
    end,
    animation = {
        speed_normal = 12,
        speed_stand = 2,
        stand_start = 2,
        stand_end = 11,
        walk_start = 12,
        walk_end = 21,
        speed_walk = 6,
        run_start = 22,
        run_end = 30,
        punch_start = 31,
        punch_end = 37,
        die_start = 39,
        die_end = 42,
        die_speed = 6,
        die_loop = false,
    },

    blood_texture = "mobs_blood.png",
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
