-- Flying Pig. Descended from "dmobs" version.

-- ===================================================================

local lcname     = "flying_pig"
local ucname     = "Flying Pig"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  40000       ,

    day_mode     = true         ,
    min_light    =      0       ,
    max_light    =     10       ,
    min_height   = -31000       ,
    max_height   =    500       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "air"                   ,
    } ,

    add_egg   = true            ,
    egg_image = "wool_pink.png" ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound = true  ,
    explosion_radius     = 3     ,

    armor         = 130          ,
    attack_type   = "explode"    ,
    damage        =   2          ,
    fall_speed    =   0          ,
    fly           = true         ,
    hp_max        =  22          ,
    hp_min        =  12          ,
    jump          = true         ,
    passive       = true         ,
    reach         =   4          ,
    runaway       = false        ,
    run_chance    =  20          ,
    run_velocity  =   3          ,
    stepheight    =   1.5        ,
    view_range    =  14          ,
    walk_velocity =   2          ,

    lava_damage   =   2          ,
    light_damage  =   0          ,
    water_damage  =   0          ,

    blood_texture = "mobs_blood.png"               ,
    collisionbox  = { -0.4, 0, -0.4, 0.4, 1, 0.4 } ,
    mesh          = msname .. ".b3d"               ,
    textures      = {{ msname .. ".png" }}         ,
    visual        = "mesh"                         ,
    visual_size   = { x=1, y=1 }                   ,

    animation     = {
        walk_start   =  1 ,
        walk_end     = 20 ,
        stand_start  =  1 ,
        stand_end    = 20 ,
        run_start    = 22 ,
        run_end      = 28 ,
        speed_normal = 10 ,
        speed_run    = 15 ,
    } ,

    drops         = {
        { name="mobs:meat_raw", chance=2, min=1, max=1 } ,
    } ,

    follow        = { "default:apple" } ,

    sounds        = {
        random  = msname        ,
        explode = "tnt_explode" ,
    } ,

    do_custom     = function (self)
        if self.state == "attack" then
            self.fly = false
            self.fall_speed = -4
        end
    end ,

    on_rightclick = function (self, clicker)
        if mobs:feed_tame (self, clicker, 8, true, true) then
            return
        end
        mobs:capture_mob (self, clicker, 0, 5, 50, false, nil)
    end ,
}

-- ===================================================================

local is_enabled = codermobs_enable_thismob
codermobs_enable_thismob = false
codermobs.setup_mob()

if is_enabled then
mobs:spawn_specific (obj_name, mobs_param.spawn_nodes,
    {
        "default:leaves"        ,
        "default:oak_leaves"    ,
        "default:pine_needles"  ,
        "default:spruce_leaves" ,
    } ,
    mobs_param.min_light        ,
    mobs_param.max_light        ,
    30                          ,
    mobs_param.spawn_chance     ,
    mobs_param.aoc              ,
    mobs_param.min_height       ,
    mobs_param.max_height)
end

codermobs.log_done()

-- ===================================================================
-- End of file.
