-- Snowman.

-- ===================================================================

local lcname     = "snowman"
local ucname     = "Snowman"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance =  20000       ,

    day_mode     = true         ,
    min_light    =      5       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt_with_snow"        ,
        "default:snow"                  ,
        "default:ice"                   ,
        "earthgen:dirt_with_dry_grass"  ,
    } ,

    add_egg   = true                    ,
    egg_image = "default_stone.png"     ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,

    visual = "mesh",
    mesh = msname .. ".b3d",
    textures = {
        {"codermobs_coal_block.png", "codermobs_coal_block.png",
        msname .. ".png", "codermobs_wool_orange.png"}
    } ,

    collisionbox = {-0.4, -0.01, -0.4, 0.4, 1, 0.4},

    animation = {
        speed_normal = 24,  speed_run = 24,
        stand_start = 41,   stand_end = 90,
        walk_start = 1,     walk_end = 40,
        run_start = 1,      run_end = 40
    } ,

    makes_footstep_sound = true,
    hp_min = 3,
    hp_max = 5,
    armor = 200,
    knock_back = 1,
    water_damage = 5,
    floats = 0,
    runaway = true,
    lava_damage = 5,
    fall_damage = 5,
    passive = false,
    attacks_monsters = true,
    view_range = 10,
    walk_velocity = 0.5,
    run_velocity = 0.5,
    stepheight    = 1.2         ,
    jump_height   =   0         ,
    jump = false,

--      drops = {
--          {name = "default:snow", chance = 1, min = 5, max = 15}
--      },
--      replace_rate = 50,
--      replace_what = {"air"},
--      replace_with = "default:snow"
}

minetest.register_node("codermobs:tophat", {
    description = "Top Hat",
    tiles = {"codermobs_coal_block.png"},
    groups = {snappy=2,choppy=2,oddly_breakable_by_hand=3,flammable=3,wool=1},
--  sounds = default_stone_sounds,
        sounds = default.node_sound_defaults() ,
    paramtype = "light",
    drawtype = "nodebox",
    node_box = {
        type = "fixed",
        fixed = {
                {-0.25, -0.5, -0.25, 0.25, -0.4375, 0.25},
                {-0.125, -0.4375, -0.125, 0.125, -0.0625, 0.125}
            }
    },
    after_place_node = function(pos, placer, itemstack, pointed_thing)
        local p = pos
        for i = 1,3 do
            p.y = p.y - 1
            local name = minetest.get_node(p).name
            if name ~= "default:snowblock" then
                return true
            end
        end
        for i = 1,4 do
            minetest.remove_node(p)
            p.y = p.y + 1
        end
        p.y = p.y - 4
        minetest.add_entity(p, obj_name)
        return true
    end
})

minetest.register_craft({
    output = 'codermobs:tophat',
    recipe = {
        {'', 'wool:black', ''},
        {'', 'wool:black', ''},
        {'wool:black', 'wool:black', 'wool:black'},
    }
})

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
