-- Wolf. Descended from "pmobs" version.

-- ===================================================================

local lcname     = "wolf"
local ucname     = "Wolf"
local msname     = "codermobs_" .. lcname
local obj_name   = "codermobs:" .. lcname

-- ===================================================================

mobs_param = {
    lcname       = lcname       ,
    ucname       = ucname       ,
    obj_name     = obj_name     ,

    aoc          =     -1       ,
    spawn_chance = 150000       ,

    day_mode     = true         ,
    min_light    =      0       ,
    max_light    =     30       ,
    min_height   = -31000       ,
    max_height   =  31000       ,

    spawn_type   = "animal"     ,

    spawn_nodes  = {
        "default:dirt"                      ,
        "default:dirt_with_dry_grass"       ,
        "default:dirt_with_grass"           ,
        "default:snow"                      ,
        "default:snowblock"                 ,
        "earthgen:dirt_with_dry_grass"      ,
        "ethereal:green_dirt"               ,
        "ethereal:green_dirt_top"           ,
        "lottmapgen:gondor_grass"           ,
        "lottmapgen:ironhill_grass"         ,
        "lottmapgen:ithilien_grass"         ,
        "lottmapgen:lorien_grass"           ,
        "lottmapgen:mirkwood_grass"         ,
        "lottmapgen:rohan_grass"            ,
        "mg:dirt_with_dry_grass"            ,
        "noisegrid:grass"                   ,
    } ,

    add_egg   = true                        ,
    egg_image = "codermobs_fur_grey.png"    ,
}

-- ===================================================================

codermobs.adjust_param()

-- ===================================================================

mobs_param.core_param = {
    type = mobs_param.spawn_type ,
    makes_footstep_sound = true  ,

    -- Most behavior/sight/battle
    armor          = 200         ,
    attack_animals = true        ,
    attack_type    = "dogfight"  ,
    damage         =   2         ,
    hp_max         =   5         ,
    passive        = false       ,
    runaway        = true        ,
    view_range     =   7         ,

    -- Velocities
    run_velocity   =   5.2557    ,
    walk_velocity  =   0.5806    ,

    -- Step/jump/fall parameters
    jump          = false        ,
    stepheight    =    1.2       ,
    jump_height   =    0         ,

    -- Environment damage
    lava_damage   =    5         ,
    light_damage  =    0         ,
    water_damage  =    1         ,

    -- Misc. parameters
    floats        =    0         ,

    blood_texture    = "mobs_blood.png"                          ,
    collisionbox     = {-0.18, 0.00, -0.18, 0.18, 0.92, 0.18}    ,
    -- drawtype         = "front"                                   ,
    rotate           = 180                                       ,
    mesh             = "codermobs_dog.b3d"                       ,
    textures         = {{ msname .. ".png" }}                    ,
    visual           = "mesh"                                    ,

    animation = {
        speed_normal = 12,
        speed_stand = 2,
        stand_start = 2,
        stand_end = 11,
        walk_start = 12,
        walk_end = 21,
        speed_walk = 6,
        run_start = 22,
        run_end = 30,
        punch_start = 31,
        punch_end = 37,
        die_start = 39,
        die_end = 42,
        die_speed = 6,
        die_loop = false,
    },

    drops            = {
        { name="mobs:meat_raw", chance=1, min=2, max=3 } ,
    } ,

    sounds           = {
        war_cry = msname .. "_attack" ,
    } ,

    specific_attack  = codermobs.make_mob_list ({
        "sheep_black"  , "sheep_blue"       , "sheep_brown"     ,
        "sheep_cyan"   , "sheep_dark_green" , "sheep_dark_grey" ,
        "sheep_green"  , "sheep_grey"       , "sheep_magenta"   ,
        "sheep_orange" , "sheep_pink"       , "sheep_red"       ,
        "sheep_violet" , "sheep_white"      , "sheep_yellow"    ,
    }) ,

    on_rightclick    = function (self, clicker)
        tool = clicker:get_wielded_item()
        local dog
        local ent
        if tool:get_name() == "mobs:meat_raw" then
            clicker:get_inventory():remove_item("main", "mobs:meat_raw")
            dog = minetest.add_entity(self.object:getpos(),
                "codermobs:dog")
            ent = dog:get_luaentity()
            ent.owner = clicker:get_player_name()
            self.object:remove()
        end
    end ,
}

-- ===================================================================

codermobs.setup_mob()
codermobs.log_done()

-- ===================================================================
-- End of file.
